from pathlib import Path

import mnist
import numpy as np
import pytest

root = Path(__file__).parent.parent.parent.parent


@pytest.fixture
def mnist_onnx() -> Path:
    return root / "tests" / "data" / "mnist_8.onnx"


@pytest.fixture
def named_tensors_onnx() -> Path:
    return root / "tests" / "data" / "named_tensors.onnx"


@pytest.fixture
def quantized_conv_truncated_onnx() -> Path:
    return root / "tests" / "data" / "quantized_i8_conv_truncated.onnx"


@pytest.fixture
def mnist_images():
    return mnist.train_images().reshape((60000, 1, 28, 28)).astype(np.float32)



# Tests for legacy API `Session`
import os
import random
from textwrap import dedent
from typing import List

import numpy as np
import pytest

from furiosa.runtime import errors, is_legacy, session, tensor


def test_run(mnist_onnx, mnist_images):
    with session.create(model=mnist_onnx) as runner:
        summary = dedent(
            """\
            Inputs:
            {0: TensorDesc(name="Input3", shape=(1, 1, 28, 28), dtype=FLOAT32, format=NCHW, size=3136, len=784)}
            Outputs:
            {0: TensorDesc(name="Plus214_Output_0", shape=(1, 10), dtype=FLOAT32, format=??, size=40, len=10)}
        """  # noqa: E501
        ).strip()

        assert runner.summary() == summary

        count = 50
        for _ in range(count):
            idx = random.randrange(0, 9999, 1)
            images = mnist_images[idx : idx + 1]

            assert images.shape == runner.input(0).shape
            assert images.dtype == runner.input(0).dtype.numpy_dtype

            result1 = runner.run(images)
            result2 = runner.run([images])
            result3 = runner.run_with(["Plus214_Output_0"], {"Input3": images})

            assert np.array_equal(result1.numpy(), result3.numpy())
            assert np.array_equal(result1.numpy(), result2.numpy())


@pytest.mark.skipif(is_legacy, reason="Legacy runtime has segfault buffer is invalid after unpack")
def test_buffer_lifetime(mnist_onnx, mnist_images):
    with session.create(model=mnist_onnx) as runner:
        # Try to unpack TensorArray to validate Tensor, TensorArray lifetime
        input1, *_ = runner.run(mnist_images[0:1])
        input1.view()


def test_run_invalid_input(mnist_onnx):
    with session.create(model=mnist_onnx) as runner:
        with pytest.raises(errors.InvalidInput):
            runner.run(np.zeros((1, 28, 28, 1), dtype=np.float16))

        with pytest.raises(errors.InvalidInput):
            runner.run(np.zeros((2, 28, 28, 2), dtype=runner.input(0).dtype.numpy_dtype))

        assert runner.run([np.zeros((1, 1, 28, 28), np.float32)])

        with pytest.raises(errors.InvalidInput):
            runner.run([np.zeros((1, 28, 28, 1), np.uint8), np.zeros((1, 28, 28, 1), np.uint8)])

        # NOTE: Nux returns InvalidInput while furiosa-native-runtime returns TypeError here
        with pytest.raises((TypeError, errors.InvalidInput)):
            runner.run_with(None, {"Input3": np.zeros((1, 28, 28, 1), np.uint8)})


def test_run_with(named_tensors_onnx):
    def assert_named_tensors(runner, input_indices: List[int], output_indices: List[int]):
        inputs = [runner.input(i) for i in input_indices]
        output_names = [runner.output(i).name for i in output_indices]
        outputs = runner.run_with(
            output_names,
            {
                inputs[0].name: tensor.rand(inputs[0]),
                inputs[1].name: tensor.rand(inputs[1]),
                inputs[2].name: tensor.rand(inputs[2]),
            },
        )

        for i in range(len(outputs)):
            assert runner.output(output_indices[i]).shape == outputs[i].shape
            assert runner.output(output_indices[i]).dtype == outputs[i].dtype

    with session.create(named_tensors_onnx) as runner:
        expected = ["input.1", "input.3", "input"]
        assert runner.input_num == 3
        for i in range(3):
            assert runner.input(i).name == expected[i]

        assert runner.output_num == 3
        expected = ["18_dequantized", "19_dequantized", "15_dequantized"]
        for i in range(3):
            assert runner.output(i).name == expected[i]

        assert_named_tensors(runner, [0, 1, 2], [0, 1, 2])
        assert_named_tensors(runner, [2, 0, 1], [0, 1, 2])
        assert_named_tensors(runner, [1, 2, 0], [0, 1, 2])

        assert_named_tensors(runner, [2, 1, 0], [0, 2, 1])
        assert_named_tensors(runner, [2, 1, 0], [1, 2, 0])
        assert_named_tensors(runner, [2, 1, 0], [2, 0, 1])
        assert_named_tensors(runner, [2, 1, 0], [2, 1, 0])


def test_run_with_invalid_names(mnist_onnx):
    with session.create(mnist_onnx) as runner:
        assert runner.input(0).name == "Input3"
        assert runner.output(0).name == "Plus214_Output_0"

        runner.run_with(["Plus214_Output_0"], {"Input3": tensor.zeros(runner.input(0))})

        with pytest.raises(errors.TensorNameNotFound):
            runner.run_with(["WrongOutput"], {"Input3": tensor.zeros(runner.input(0))})

        # Wrong input
        with pytest.raises(errors.TensorNameNotFound):
            runner.run_with(["Plus214_Output_0"], {"WrongInput3": tensor.zeros(runner.input(0))})


@pytest.mark.skipif(os.getenv("NPU_DEVNAME") is None, reason="No NPU_DEVNAME defined")
def test_device_busy(mnist_onnx):
    runner = session.create(mnist_onnx, device=os.getenv("NPU_DEVNAME"))

    with pytest.raises(errors.DeviceBusy):
        session.create(mnist_onnx, device=os.getenv("NPU_DEVNAME"))

    runner.close()


def test_closed(mnist_onnx):
    runner = session.create(mnist_onnx)
    runner.close()

    with pytest.raises(errors.SessionClosed):
        runner.inputs()
    with pytest.raises(errors.SessionClosed):
        runner.input_num()
    with pytest.raises(errors.SessionClosed):
        runner.outputs()
    with pytest.raises(errors.SessionClosed):
        runner.output_num()
    with pytest.raises(errors.SessionClosed):
        runner.summary()
    with pytest.raises(errors.SessionClosed):
        runner.run(None)


def test_create(mnist_onnx):
    runner = session.create(
        mnist_onnx, worker_num=1, compiler_config={"allow_precision_error": True}
    )
    assert runner
    runner.close()


def test_batch(mnist_onnx):
    with session.create(mnist_onnx, batch_size=1) as runner:
        assert runner.input(0).shape == (1, 1, 28, 28)

    with session.create(mnist_onnx, batch_size=4) as runner:
        assert runner.input(0).shape == (4, 1, 28, 28)


def test_invalid_config(mnist_onnx):
    invalid_config = {"remove_lower": 1}  # Correct value type is boolean

    # FIXME: Add specific compiler errors to furiosa-native-runtime
    with pytest.raises(Exception):
        session.create(mnist_onnx, compiler_config=invalid_config)



import pytest

from furiosa.runtime import is_legacy

if is_legacy:
    pytest.skip("skipping furiosa-rt tests", allow_module_level=True)


def test_utils():
    from furiosa.runtime import Axis, DataType, Model, ModelSource, Tensor, TensorArray, TensorDesc


def test_runner():
    from furiosa.runtime.sync import Runner, Runtime, create_runner


def test_async_runner():
    from furiosa.runtime import Runner, Runtime, create_runner


def test_queue_api():
    from furiosa.runtime import Receiver, Submitter, create_queue


def test_sync_queue_api():
    from furiosa.runtime.sync import Receiver, Submitter, create_queue


def test_profiler():
    from furiosa.runtime.profiler import RecordFormat, Resource, profile


def test_diagnostics():
    from furiosa.runtime import FuriosaRuntimeError, FuriosaRuntimeWarning, __full_version__


def test_legacy_session_and_queue_api():
    with pytest.warns(FutureWarning):
        from furiosa.runtime.session import (
            AsyncSession,
            CompletionQueue,
            Session,
            create,
            create_async,
        )


def test_legacy_support():
    with pytest.warns(FutureWarning):
        from furiosa.runtime.envs import (
            current_npu_device,
            is_compile_log_enabled,
            log_dir,
            profiler_output,
        )



# Tests for legacy API `AsyncSession`
import os
from pathlib import Path
import random
from textwrap import dedent

import pytest

from furiosa.runtime import errors, session


def test_run_async(mnist_onnx, mnist_images):
    submitter, receiver = session.create_async(model=Path(mnist_onnx), input_queue_size=50)

    summary = dedent(
        """\
        Inputs:
        {0: TensorDesc(name="Input3", shape=(1, 1, 28, 28), dtype=FLOAT32, format=NCHW, size=3136, len=784)}
        Outputs:
        {0: TensorDesc(name="Plus214_Output_0", shape=(1, 10), dtype=FLOAT32, format=??, size=40, len=10)}
    """  # noqa: E501
    ).strip()
    assert submitter.summary() == summary

    count = 50
    for i in range(count):
        j = random.randrange(0, 9999, 1)
        images = mnist_images[j : j + 1]

        assert images.shape == submitter.input(0).shape
        assert images.dtype == submitter.input(0).dtype.numpy_dtype

        submitter.submit([images], i)

    assert {next(receiver)[0] for _ in range(count)} == set(range(count))

    submitter.close()
    receiver.close()


def test_create(mnist_onnx):
    submitter, receiver = session.create_async(
        mnist_onnx,
        worker_num=1,
        input_queue_size=1,
        output_queue_size=1,
        compiler_config={"allow_precision_error": True},
    )

    submitter.close()
    receiver.close()


def test_timeout(mnist_onnx):
    submitter, receiver = session.create_async(model=mnist_onnx)

    with pytest.raises(errors.QueueWaitTimeout):
        receiver.recv(timeout=int(0))
    with pytest.raises(errors.QueueWaitTimeout):
        receiver.recv(timeout=100)

    submitter.close()

    with pytest.raises(errors.SessionTerminated):
        receiver.recv()
    with pytest.raises(errors.SessionTerminated):
        receiver.recv(timeout=0)
    with pytest.raises(errors.SessionTerminated):
        receiver.recv(timeout=100)

    receiver.close()


def test_submitter_closed(mnist_onnx):
    submitter, receiver = session.create_async(mnist_onnx)
    submitter.close()

    with pytest.raises(errors.SessionClosed):
        submitter.inputs()
    with pytest.raises(errors.SessionClosed):
        submitter.input_num()
    with pytest.raises(errors.SessionClosed):
        submitter.outputs()
    with pytest.raises(errors.SessionClosed):
        submitter.output_num()
    with pytest.raises(errors.SessionClosed):
        submitter.summary()

    receiver.close()


def test_queue_closed(mnist_onnx):
    submitter, receiver = session.create_async(mnist_onnx)
    receiver.close()
    with pytest.raises(errors.SessionClosed):
        receiver.recv()

    submitter.close()


@pytest.mark.skipif(os.getenv("NPU_DEVNAME") is None, reason="No NPU_DEVNAME defined")
def test_device_busy(mnist_onnx):
    submitter, receiver = session.create_async(mnist_onnx, device=os.getenv("NPU_DEVNAME"))

    with pytest.raises(errors.DeviceBusy):
        submitter, receiver = session.create_async(mnist_onnx, device=os.getenv("NPU_DEVNAME"))

    submitter.close()
    receiver.close()



import warnings

try:
    import package_extras
except ModuleNotFoundError:
    from furiosa.native_runtime.compiler import *

    warnings.warn(
        "'furiosa.runtime.compiler' module is deprecated and will be removed in a future release.",
        category=FutureWarning,
    )
else:
    from furiosa.runtime.legacy.compiler import *



import warnings

try:
    import package_extras
except ModuleNotFoundError:
    warnings.warn(
        "'furiosa.runtime.envs' module is deprecated and will be removed in a future release.",
        category=FutureWarning,
    )

# Module 'envs' is same for two implementations
from furiosa.runtime.legacy.envs import *



import os

from furiosa import runtime


def default_device() -> str:
    """Returns default device string for the current runtime.
    This function is for backward compatibility of the device argument for runtime."""
    if runtime.is_legacy:
        return os.environ.get("NPU_DEVNAME", "npu0pe0-1")
    else:
        return os.environ.get("FURIOSA_DEVICES", os.environ.get("NPU_DEVNAME", "warboy(2)*1"))



import os

from furiosa import runtime


def test_default_device():
    if runtime.is_legacy:
        # Remove NPU_DEVNAME
        os.environ.pop('NPU_DEVNAME', None)
        assert runtime._utils.default_device() == 'npu0pe0-1'
    else:
        # Remove FURIOSA_DEVICES & NPU_DEVNAME
        os.environ.pop('NPU_DEVNAME', None)
        os.environ.pop('FURIOSA_DEVICES', None)
        assert runtime._utils.default_device() == 'warboy(2)*1'



import re
import unittest

from furiosa import runtime
from furiosa.runtime import __full_version__


def test_version():
    assert len(runtime.__version__.version) > 0
    assert runtime.__version__.stage in ("dev", "release")
    assert len(runtime.__version__.hash) > 0


def test_full_version():
    # Furiosa SDK Runtime 0.10.0-dev (rev: 705853b9) (libnux 0.10.0-dev c3412f038 2023-07-11T03:15:45Z) # noqa: E501
    # Furiosa SDK Runtime 0.10.0-dev (rev: 705853b9) (furiosa-rt 0.1.0-dev 2bd01c5a9 2023-07-14T01:01:24Z) # noqa: E501
    matcher = re.compile(r"Furiosa SDK Runtime (\S+) \(rev: (\S+)\) \((\S+ \S+ \S+ \S+)\)")
    assert matcher.match(runtime.__full_version__)



import random
from textwrap import dedent
from typing import List

import numpy as np
import pytest

from furiosa.runtime import is_legacy

if is_legacy:
    pytest.skip("skipping legacy tests in test_runner", allow_module_level=True)

from furiosa.runtime import FuriosaRuntimeError, Runtime, create_runner, tensor


async def run_inner(runner, mnist_images):
    model = runner.model
    summary = dedent(
        """\
        Inputs:
        {0: TensorDesc(name="Input3", shape=(1, 1, 28, 28), dtype=FLOAT32, format=NCHW, size=3136, len=784)}
        Outputs:
        {0: TensorDesc(name="Plus214_Output_0", shape=(1, 10), dtype=FLOAT32, format=??, size=40, len=10)}
    """  # noqa: E501
    ).strip()

    assert model.summary() == summary

    count = 50
    for _ in range(count):
        idx = random.randrange(0, 9999, 1)
        images = mnist_images[idx : idx + 1]

        assert images.shape == model.input(0).shape
        assert images.dtype == model.input(0).dtype.numpy

        result1 = await runner.run(images)
        result2 = await runner.run([images])
        result3 = await runner.run_with(["Plus214_Output_0"], {"Input3": images})

        assert np.array_equal(result1, result2)
        assert np.array_equal(result1, result3)


@pytest.mark.asyncio
async def test_create_runner(mnist_onnx, mnist_images):
    async with create_runner(mnist_onnx) as runner:
        await run_inner(runner, mnist_images)


@pytest.mark.asyncio
async def test_runtime(mnist_onnx, mnist_images):
    async with Runtime() as runtime:
        async with runtime.create_runner(mnist_onnx) as runner:
            await run_inner(runner, mnist_images)


@pytest.mark.asyncio
async def test_buffer_lifetime(mnist_onnx, mnist_images):
    async with create_runner(mnist_onnx) as runner:
        # Try to unpack TensorArray to validate Tensor, TensorArray lifetime
        input1, *_ = await runner.run(mnist_images[0:1])
        input1.view()


@pytest.mark.asyncio
async def test_run_invalid_input(mnist_onnx):
    async with create_runner(mnist_onnx) as runner:
        # FIXME: Add specific error variants to furiosa-native-runtime
        with pytest.raises(TypeError):
            await runner.run(np.zeros((1, 28, 28, 1), dtype=np.float16))

        with pytest.raises(ValueError):
            await runner.run(np.zeros((2, 28, 28, 2), dtype=runner.model.input(0).dtype.numpy))

        assert await runner.run([np.zeros((1, 1, 28, 28), np.float32)])

        with pytest.raises(ValueError):
            await runner.run(
                [np.zeros((1, 28, 28, 1), np.uint8), np.zeros((1, 28, 28, 1), np.uint8)]
            )

        with pytest.raises(TypeError):
            await runner.run_with(None, {"Input3": np.zeros((1, 28, 28, 1), np.uint8)})


@pytest.mark.asyncio
async def test_run_with(named_tensors_onnx):
    async def assert_named_tensors(runner, input_indices: List[int], output_indices: List[int]):
        model = runner.model
        inputs = [model.input(i) for i in input_indices]
        output_names = [model.output(i).name for i in output_indices]
        outputs = await runner.run_with(
            output_names,
            {
                inputs[0].name: tensor.rand(inputs[0]),
                inputs[1].name: tensor.rand(inputs[1]),
                inputs[2].name: tensor.rand(inputs[2]),
            },
        )

        for i in range(len(outputs)):
            assert model.output(output_indices[i]).shape == outputs[i].shape
            assert model.output(output_indices[i]).dtype.numpy == outputs[i].dtype

    async with create_runner(named_tensors_onnx) as runner:
        model = runner.model
        expected = ["input.1", "input.3", "input"]
        assert model.input_num == 3
        for i in range(3):
            assert model.input(i).name == expected[i]

        assert model.output_num == 3
        expected = ["18_dequantized", "19_dequantized", "15_dequantized"]
        for i in range(3):
            assert model.output(i).name == expected[i]

        await assert_named_tensors(runner, [0, 1, 2], [0, 1, 2])
        await assert_named_tensors(runner, [2, 0, 1], [0, 1, 2])
        await assert_named_tensors(runner, [1, 2, 0], [0, 1, 2])

        await assert_named_tensors(runner, [2, 1, 0], [0, 2, 1])
        await assert_named_tensors(runner, [2, 1, 0], [1, 2, 0])
        await assert_named_tensors(runner, [2, 1, 0], [2, 0, 1])
        await assert_named_tensors(runner, [2, 1, 0], [2, 1, 0])


@pytest.mark.asyncio
async def test_run_with_invalid_names(mnist_onnx):
    async with create_runner(mnist_onnx) as runner:
        model = runner.model
        assert model.input(0).name == "Input3"
        assert model.output(0).name == "Plus214_Output_0"

        await runner.run_with(["Plus214_Output_0"], {"Input3": tensor.zeros(model.input(0))})

        with pytest.raises(ValueError):
            await runner.run_with(["WrongOutput"], {"Input3": tensor.zeros(model.input(0))})

        # Wrong input
        with pytest.raises(ValueError):
            await runner.run_with(
                ["Plus214_Output_0"], {"WrongInput3": tensor.zeros(model.input(0))}
            )


@pytest.mark.asyncio
async def test_device_busy(mnist_onnx):
    runner = await create_runner(mnist_onnx)

    # FIXME: Add specific error variants to furiosa-native-runtime
    with pytest.raises(FuriosaRuntimeError):
        await create_runner(mnist_onnx)

    assert await runner.close()


@pytest.mark.asyncio
async def test_closed(mnist_onnx):
    runner = await create_runner(mnist_onnx)
    assert await runner.close()

    with pytest.raises(FuriosaRuntimeError):
        await runner.run([np.zeros((1, 1, 28, 28), np.float32)])


@pytest.mark.asyncio
async def test_create(mnist_onnx):
    runner = await create_runner(
        mnist_onnx, worker_num=1, compiler_config={"allow_precision_error": True}
    )
    assert runner
    assert await runner.close()


@pytest.mark.asyncio
async def test_batch(mnist_onnx):
    async with create_runner(mnist_onnx, batch_size=1) as runner:
        assert runner.model.input(0).shape == (1, 1, 28, 28)

    async with create_runner(mnist_onnx, batch_size=4) as runner:
        assert runner.model.input(0).shape == (4, 1, 28, 28)


@pytest.mark.asyncio
async def test_invalid_config_name(mnist_onnx):
    invalid_config = {"INVALID_COMPILER_CONFIG_NAME": True}
    # FIXME: Add specific error variants to furiosa-native-runtime
    with pytest.raises(FuriosaRuntimeError):
        await create_runner(mnist_onnx, worker_num=1, compiler_config=invalid_config)


@pytest.mark.asyncio
async def test_invalid_config_type(mnist_onnx):
    invalid_config = {"remove_lower": 1}  # Correct value type is boolean
    # FIXME: Add specific error variants to furiosa-native-runtime
    with pytest.raises(FuriosaRuntimeError):
        await create_runner(mnist_onnx, worker_num=1, compiler_config=invalid_config)



import random

import numpy as np

from furiosa.runtime import session
from furiosa.runtime.tensor import DataType, numpy_dtype


def test_dtype(mnist_onnx):
    with session.create(mnist_onnx) as runner:
        assert np.float32 == numpy_dtype(runner.input(0).dtype)


def test_tensor_desc(mnist_onnx):
    with session.create(mnist_onnx) as runner:
        tensor = runner.input(0)
        assert tensor.ndim == 4
        assert tensor.format == "NCHW"
        assert tensor.shape == (1, 1, 28, 28)
        assert tensor.dtype == DataType.FLOAT32
        assert numpy_dtype(tensor) == np.float32
        assert tensor.length == 784
        assert tensor.size == 3136

    # FIXME: Add named tensor check


def assert_tensors_equal(expected, result):
    assert np.allclose(expected, result, atol=1.0), "{} was expected, but the result was {}".format(
        expected, result
    )


def test_tensor(mnist_onnx, mnist_images):
    with session.create(mnist_onnx) as runner:
        assert runner.input_num == 1
        assert runner.output_num == 1

        inputs = runner.allocate_inputs()
        inputs2 = runner.allocate_inputs()

        assert len(inputs) == 1
        assert len(inputs[0:1]) == 1
        assert numpy_dtype(inputs[0]) == np.float32

        # FIXME: Add named tensor check

        for i in range(10):
            idx = random.randrange(0, 9999, 1)
            images = mnist_images[idx : idx + 1]

            inputs[0] = images

            assert np.allclose(images, inputs[0].numpy())

            # Idempotent
            assert inputs[0] == inputs[0]

            # Equality of contents
            inputs2[0] = images
            assert inputs[0] == inputs2[0]


def test_tensor_names(named_tensors_onnx):
    with session.create(named_tensors_onnx) as runner:
        assert runner.input_num == 3
        assert runner.output_num == 3

        # FIXME: Add named tensor check


def test_lowered_tensor_desc(quantized_conv_truncated_onnx):
    with session.create(
        quantized_conv_truncated_onnx,
        compiler_config={
            "remove_unlower": True,
        },
    ) as runner:
        tensor = runner.output(0)

        assert tensor.ndim == 6
        assert tensor.dtype == DataType.INT8
        assert numpy_dtype(tensor) == np.int8
        assert tensor.size > tensor.length * np.dtype(tensor.numpy_dtype).itemsize


def test_lowered_tensor_numpy(quantized_conv_truncated_onnx):
    with session.create(
        quantized_conv_truncated_onnx,
        compiler_config={
            "remove_unlower": True,
        },
    ) as runner:
        tensor = runner.output(0)

        strides = []
        stride = 1
        for axis in reversed(range(tensor.ndim)):
            strides.append(stride)
            stride *= tensor.dim(axis)
        strides = tuple(reversed(strides))

        inputs = []
        for input in runner.inputs():
            inputs.append(np.random.random(input.shape).astype(np.float32))

        outputs = runner.run(inputs)

        output = outputs[0].view()

        assert output.ndim == 6
        assert numpy_dtype(output) == np.int8
        assert output.strides != strides

        # Calling numpy() rearranges the memory layout due to internal copy() call
        output = outputs[0].numpy()

        assert output.ndim == 6
        assert numpy_dtype(output) == np.int8
        assert output.strides == strides



"""Provide high-level Python APIs to access Furiosa AI's NPUs and its eco-system"""

import warnings

from furiosa.common.utils import get_sdk_version

from . import _utils

try:
    import package_extras
except ModuleNotFoundError:
    try:
        from furiosa.native_runtime import *
    except ImportError as e:
        raise e from None

    def full_version() -> str:
        """Returns a full version from furiosa-rt version"""
        import furiosa.native_runtime as rt

        return "Furiosa SDK Runtime {} (furiosa-rt {} {} {})".format(
            __version__,
            rt.__version__,
            rt.__git_short_hash__,
            rt.__build_timestamp__,
        )
        return __full_version__

    is_legacy = False
else:
    warnings.warn(
        "'furiosa-runtime[legacy]' is deprecated and will be removed in a future release.",
        category=FutureWarning,
    )

    def full_version() -> str:
        """Returns a full version string including the Nux version"""

        from .legacy._api import LIBNUX as rt

        return "Furiosa SDK Runtime {} (libnux {} {} {})".format(
            __version__,
            rt.version().decode("utf-8"),
            rt.git_short_hash().decode("utf-8"),
            rt.build_timestamp().decode("utf-8"),
        )

    is_legacy = True

    del package_extras


__version__ = get_sdk_version(__name__)
__full_version__ = full_version()

del warnings, get_sdk_version, full_version


__all__ = ["__version__", "__full_version__", "is_legacy"]



try:
    import package_extras
except ModuleNotFoundError:
    from furiosa.native_runtime.bench import *
else:
    # furiosa.runtime.bench is not available in legacy
    raise ModuleNotFoundError("No module named 'furiosa.runtime.bench")



import pytest

from furiosa.runtime import is_legacy

if not is_legacy:
    pytest.skip("skipping legacy tests", allow_module_level=True)


def test_legacy_session():
    from furiosa.runtime.session import Session, create


def test_legacy_support():
    from furiosa.runtime.envs import (
        current_npu_device,
        is_compile_log_enabled,
        log_dir,
        profiler_output,
    )
